@@ -9,15 +9,26 @@ except NameError: # python 3
 def upFirstLetter(s):
     return s[0].upper() + s[1:]
 
+def stringArray(strings):
+    return "[NSArray arrayWithObjects:%s,nil]" % ','.join(('@"%s"' % s) for s in strings)
+
+globalLocalizationTable = None
+
 def convertValueToObjc(value):
-    if isinstance(value, GeneratedItem):
+    if value is None:
+        return 'nil'
+    elif isinstance(value, GeneratedItem):
         return value.varname
     elif isinstance(value, KeyValueId):
         return value._objcAccessor()
     elif isinstance(value, Literal):
         return value.value
     elif isinstance(value, basestring):
-        return '@"%s"' % value
+        value = value.replace('\n', '\\n')
+        result = '@"%s"' % value
+        if value and globalLocalizationTable:
+            result = 'NSLocalizedStringFromTable(%s, @"%s", @"")' % (result, globalLocalizationTable)
+        return result
     elif isinstance(value, bool):
         return 'YES' if value else 'NO'
     elif isinstance(value, (int, float)):
@@ -47,7 +58,7 @@ class CodeTemplate(object):
             # strings contain replacement placeholders. We want to perform replacements on those
             # strings too.
             for placeholder in placeholders:
-                replacement = replacements.get(placeholder[1:-1], '')
+                replacement = str(replacements.get(placeholder[1:-1], ''))
                 result = result.replace(placeholder, replacement)
             placeholders = re.findall(r"\$\w+?\$", result)
         return result
@@ -67,6 +78,9 @@ class KeyValueId(object):
         self._fakeParent = fakeParent
         self._children = {}
     
+    def __repr__(self):
+        return '<KeyValueId %s>' % self._objcAccessor()
+    
     def __getattr__(self, name):
         if name.startswith('_'):
             return object.__getattribute__(self, name)
@@ -88,13 +102,23 @@ class KeyValueId(object):
     # avoid name clashes.
     def _objcAccessor(self):
         if self._parent and not self._parent._fakeParent:
-            if self.__parent._name == 'nil':
+            if self._parent._name == 'nil':
                 return 'nil'
             else:
                 return '[%s %s]' % (self._parent._objcAccessor(), self._name)
         else:
             return self._name
     
+    def _callMethod(self, methodname, argument=None, endline=True):
+        # For now, this method only supports call to methods of zero or one argument.
+        if argument is None:
+            result = getattr(self, methodname)._objcAccessor()
+        else:
+            result = '[%s %s:%s]' % (self._objcAccessor(), methodname, convertValueToObjc(argument))
+        if endline:
+            result += ';\n'
+        return result
+    
     def _clear(self):
         for child in self._children.values():
             child._clear()
@@ -152,13 +176,10 @@ class Literal(object):
     
 
 class GeneratedItem(object):
-    CREATION_ORDER_COUNTER = 0
     OBJC_CLASS = 'NSObject'
     
     def __init__(self):
-        self.creationOrder = GeneratedItem.CREATION_ORDER_COUNTER
-        GeneratedItem.CREATION_ORDER_COUNTER += 1
-        self.generated = False
+        self.creationOrder = globalGenerationCounter.creationToken()
         # In case we are never assigned to a top level variable and thus never given a varname
         self.varname = "_tmp%d" % self.creationOrder
         # properties to be set at generation time. For example, if "editable" is set to False,
@@ -170,7 +191,8 @@ class GeneratedItem(object):
         tmpl = CodeTemplate("$allocinit$\n$setup$\n$setprop$\n")
         tmpl.varname = self.varname
         tmpl.classname = self.OBJC_CLASS
-        tmpl.allocinit = "$classname$ *$varname$ = [[$classname$ alloc] init];"
+        tmpl.allocinit = "$classname$ *$varname$ = [[[$classname$ alloc] $initmethod$] autorelease];"
+        tmpl.initmethod = "init"
         tmpl.setup = ''
         return tmpl
     
@@ -179,14 +201,21 @@ class GeneratedItem(object):
         return []
     
     #--- Public
+    @property
+    def accessor(self):
+        return KeyValueId(None, self.varname)
+    
+    @property
+    def generated(self):
+        return globalGenerationCounter.isGenerated(self)
+    
     def generateAssignments(self):
         if self not in KeyValueId.VALUE2KEYS:
             return ""
         assignments = []
         for key in KeyValueId.VALUE2KEYS[self]:
-            parentAccessor = key._parent._objcAccessor()
-            setmethod = 'set' + key._name[0].upper() + key._name[1:]
-            assignment = "[%s %s: %s];" % (parentAccessor, setmethod, self.varname)
+            setmethod = 'set' + upFirstLetter(key._name)
+            assignment = key._parent._callMethod(setmethod, self)
             assignments.append(assignment)
         return '\n'.join(assignments)
     
@@ -200,11 +229,42 @@ class GeneratedItem(object):
         for key, value in self.properties.items():
             if value is None:
                 continue
-            value = convertValueToObjc(value)
-            setprop += '[$varname$ set%s:%s];\n' % (upFirstLetter(key), value)
+            dot_elements = key.split('.')
+            accessor = self.accessor
+            for dot_element in dot_elements[:-1]:
+                accessor = getattr(accessor, dot_element)
+            if isinstance(value, GeneratedItem) and not value.generated:
+                # Generate an assignment (which is generated by the "value" part of the assignment)
+                # so that we set that value after our target item was generated
+                setattr(accessor, dot_elements[-1], value)
+            else:
+                methname = 'set' + upFirstLetter(dot_elements[-1])
+                setprop += accessor._callMethod(methname, value)
         inittmpl.setprop = setprop
         result += inittmpl.render()
         result += self.generateAssignments()
-        self.generated = True
+        globalGenerationCounter.addGenerated(self)
         return result
     
+
+class GenerationCounter(object):
+    def __init__(self):
+        self.reset()
+    
+    def creationToken(self):
+        count = self.creationCount
+        self.creationCount += 1
+        return count
+    
+    def addGenerated(self, item):
+        self.generatedItems.add(item)
+    
+    def isGenerated(self, item):
+        return item in self.generatedItems
+    
+    def reset(self):
+        self.creationCount = 0
+        self.generatedItems = set()
+    
+
+globalGenerationCounter = GenerationCounter()
\ No newline at end of file
