@@ -3,16 +3,24 @@ import tempfile
 import shutil
 from subprocess import Popen
 
-from .base import CodeTemplate, KeyValueId, Action, GeneratedItem, owner, NSApp, const
-from .view import Pack
+from . import base
+from .base import CodeTemplate, Action, GeneratedItem, owner, NSApp, const
+from .view import View, Pack
+from .font import Font, FontFamily, FontSize, FontTrait
+from .color import Color
 from .menu import Menu, MainMenu
 from .window import Window
 from .button import Button, Checkbox
 from .label import Label
 from .textfield import TextField
+from .textview import TextView
 from .popup import Popup
 from .combo import Combobox
-from .font import Font, FontFamily, FontSize, FontTrait
+from .radio import RadioButtons
+from .progress import ProgressIndicator
+from .image import ImageView
+from .tabview import TabView
+from .table import TableView
 
 try:
     execfile
@@ -24,8 +32,9 @@ except NameError:
 
 UNIT_TMPL = """
 #import <Cocoa/Cocoa.h>
+$ownerimport$
 
-$classname$* create$name$($ownerclass$ *owner)
+$classname$* create$name$($ownerdecl$)
 {
 $contents$
 return result;
@@ -35,11 +44,14 @@ return result;
 # ownerless is used by runUI. When running a UI, we take one UI script out of its context, so
 # any owner assignment will make code compilation fail. Since we just want to preview the UI, we
 # don't need those assignments, so we skip them.
-def generate(modulePath, dest, ownerless=False):
+def generate(modulePath, dest, ownerless=False, localizationTable=None):
+    base.globalLocalizationTable = localizationTable
+    base.globalGenerationCounter.reset()
     module_globals = {
         'owner': owner,
         'NSApp': NSApp,
         'const': const,
+        'View': View,
         'Menu': Menu,
         'MainMenu': MainMenu,
         'Action': Action,
@@ -48,12 +60,19 @@ def generate(modulePath, dest, ownerless=False):
         'Checkbox': Checkbox,
         'Label': Label,
         'TextField': TextField,
+        'TextView': TextView,
         'Popup': Popup,
         'Combobox': Combobox,
+        'RadioButtons': RadioButtons,
+        'ProgressIndicator': ProgressIndicator,
+        'ImageView': ImageView,
+        'TabView': TabView,
+        'TableView': TableView,
         'Font': Font,
         'FontFamily': FontFamily,
         'FontSize': FontSize,
         'FontTrait': FontTrait,
+        'Color': Color,
         'Pack': Pack,
     }
     module_locals = {}
@@ -63,9 +82,17 @@ def generate(modulePath, dest, ownerless=False):
     if ownerless:
         owner._clear()
         owner._name = 'nil'
-        tmpl.ownerclass = 'id'
+        ownerclass = 'id'
+        ownerimport = None
     else:
-        tmpl.ownerclass = module_locals.get('ownerclass', 'id')
+        ownerclass = module_locals.get('ownerclass', 'id')
+        ownerimport = module_locals.get('ownerimport')
+    if ownerclass == 'id':
+        tmpl.ownerdecl = "id owner"
+    else:
+        tmpl.ownerdecl = "%s *owner" % ownerclass
+    if ownerimport:
+        tmpl.ownerimport = "#import \"%s\"" % ownerimport
     tmpl.name = os.path.splitext(os.path.basename(dest))[0]
     toGenerate = []
     for key, value in module_locals.items():
@@ -92,7 +119,7 @@ def runUI(modulePath):
     tmpPath = tempfile.mkdtemp()
     destPath = os.path.join(tmpPath, 'runtemplate')
     shutil.copytree(runtemplatePath, destPath)
-    shutil.copy(modulePath, os.path.join(destPath, 'MainWindow.py'))
+    shutil.copy(modulePath, os.path.join(destPath, 'MainScript.py'))
     cmd = 'cd "%s" && python ./waf configure && python ./waf && open build/RunUI.app -W && cd ../.. && rm -r "%s"' % (destPath, tmpPath)
     p = Popen(cmd, shell=True)
     p.wait()
